@@ -15,12 +15,13 @@
 
 #include "bonobo-property-editor.h"
 
-#define CLASS(o) BONOBO_PROPERTY_EDITOR_CLASS (GTK_OBJECT(o)->klass)
+#define CLASS(o) BONOBO_PEDITOR_CLASS (GTK_OBJECT(o)->klass)
 
-struct _BonoboPropertyEditorPrivate {
-	Bonobo_Property               property;
-	BonoboPropertyEditorSetFn     set_cb;
-	Bonobo_EventSource_ListenerId id;
+struct _BonoboPEditorPrivate {
+	Bonobo_Property    property;
+	BonoboPEditorSetFn set_cb;
+	Bonobo_EventSource_ListenerId  id;
+	GtkWidget          *widget;
 };
 
 #define  TYPE_PRINT(tckind)			                        \
@@ -101,7 +102,8 @@ print_typecode (CORBA_TypeCode tc, const gchar *name, int level)
 }
 
 static void
-int_set_value (BonoboPropertyEditor *editor, CORBA_any *any,
+int_set_value (BonoboPEditor     *editor, 
+	       CORBA_any         *any,
 	       CORBA_Environment *ev)
 {
 	CORBA_any nv;
@@ -133,7 +135,7 @@ value_changed_cb (BonoboListener    *listener,
 		  CORBA_Environment *ev,
 		  gpointer           user_data)
 {
-	BonoboPropertyEditor *editor = BONOBO_PROPERTY_EDITOR (user_data);
+	BonoboPEditor *editor = BONOBO_PEDITOR (user_data);
 	
 	if (!bonobo_arg_type_is_equal (any->_type, editor->tc, ev)) {
 		bonobo_exception_set 
@@ -147,11 +149,11 @@ value_changed_cb (BonoboListener    *listener,
 }
 
 void                
-bonobo_property_editor_set_property (BonoboPropertyEditor *editor,
-				     Bonobo_PropertyBag    bag,
-				     const char           *name,
-				     CORBA_TypeCode        tc, 
-				     CORBA_any            *defval)
+bonobo_peditor_set_property (BonoboPEditor      *editor,
+			     Bonobo_PropertyBag  bag,
+			     const char         *name,
+			     CORBA_TypeCode      tc, 
+			     CORBA_any          *defval)
 {
 	CORBA_Environment  ev;
 	Bonobo_Property    property;
@@ -223,9 +225,9 @@ bonobo_property_editor_set_property (BonoboPropertyEditor *editor,
 
 
 static void
-bonobo_property_editor_destroy (GtkObject *object)
+bonobo_peditor_destroy (GtkObject *object)
 {
-	BonoboPropertyEditor *editor = BONOBO_PROPERTY_EDITOR (object);
+	BonoboPEditor *editor = BONOBO_PEDITOR (object);
 	CORBA_Environment ev;
 	
 	CORBA_exception_init (&ev);
@@ -247,115 +249,91 @@ bonobo_property_editor_destroy (GtkObject *object)
 }
 
 static void 
-real_property_editor_set_value (BonoboPropertyEditor *editor,
-				BonoboArg            *value,
-				CORBA_Environment    *ev)
+real_peditor_set_value (BonoboPEditor     *editor,
+			BonoboArg         *value,
+			CORBA_Environment *ev)
 {
 	g_warning ("calling default set_value implementation");
 }
 
 static void
-pe_size_request (GtkWidget      *widget,
-		 GtkRequisition *requisition)
-{
-	GtkBin *bin = GTK_BIN (widget);
-
-	if (bin->child && GTK_WIDGET_VISIBLE (bin->child)) 
-		gtk_widget_size_request (bin->child, requisition);
-}
-
-static void
-pe_size_allocate (GtkWidget     *widget,
-		  GtkAllocation *allocation)
-{
-	GtkBin *bin = GTK_BIN (widget);
-
-	widget->allocation = *allocation;
-
-	if (bin->child && GTK_WIDGET_VISIBLE (bin->child)) 
-		gtk_widget_size_allocate (bin->child, allocation);
-}
-
-static void
-bonobo_property_editor_class_init (BonoboPropertyEditorClass *klass)
+bonobo_peditor_class_init (BonoboPEditorClass *klass)
 {
 	GtkObjectClass *object_class = (GtkObjectClass *)klass;
-	GtkWidgetClass *widget_class = (GtkWidgetClass *)klass;
 
-
-	klass->set_value = real_property_editor_set_value;
-
-	widget_class->size_request = pe_size_request;
-	widget_class->size_allocate = pe_size_allocate;
+	klass->set_value = real_peditor_set_value;
   
-	object_class->destroy = bonobo_property_editor_destroy;
+	object_class->destroy = bonobo_peditor_destroy;
 }
 
 static void
-bonobo_property_editor_init (BonoboPropertyEditor *editor)
+bonobo_peditor_init (BonoboPEditor *editor)
 {
 	GTK_WIDGET_SET_FLAGS (editor, GTK_CAN_FOCUS);
 	GTK_WIDGET_SET_FLAGS (editor, GTK_NO_WINDOW);
 
-	editor->priv = g_new0 (BonoboPropertyEditorPrivate, 1);
+	editor->priv = g_new0 (BonoboPEditorPrivate, 1);
 }
 
 GtkType
-bonobo_property_editor_get_type (void)
+bonobo_peditor_get_type (void)
 {
 	static GtkType petype = 0;
 
 	if (!petype) {
 		static const GtkTypeInfo peinfo = {
-			"BonoboPropertyEditor",
-			sizeof (BonoboPropertyEditor),
-			sizeof (BonoboPropertyEditorClass),
-			(GtkClassInitFunc) bonobo_property_editor_class_init,
-			(GtkObjectInitFunc) bonobo_property_editor_init,
+			"BonoboPEditor",
+			sizeof (BonoboPEditor),
+			sizeof (BonoboPEditorClass),
+			(GtkClassInitFunc) bonobo_peditor_class_init,
+			(GtkObjectInitFunc) bonobo_peditor_init,
 			/* reserved_1 */ NULL,
 			/* reserved_2 */ NULL,
 			(GtkClassInitFunc) NULL,
 		};
 
-		petype = gtk_type_unique (GTK_TYPE_BIN, &peinfo);
+		petype = gtk_type_unique (GTK_TYPE_OBJECT, &peinfo);
 	}
 
 	return petype;
 }
 
-BonoboPropertyEditor *
-bonobo_property_editor_construct (GtkWidget                 *widget, 
-				  BonoboPropertyEditorSetFn  set_cb,
-				  CORBA_TypeCode             tc)
+BonoboPEditor *
+bonobo_peditor_construct (GtkWidget          *widget, 
+			  BonoboPEditorSetFn  set_cb,
+			  CORBA_TypeCode      tc)
 {
-	BonoboPropertyEditor *editor;
+	BonoboPEditor *editor;
 	
 	g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 	g_return_val_if_fail (set_cb != NULL, NULL);
 
-	editor = gtk_type_new (bonobo_property_editor_get_type ());
-
-	gtk_container_add (GTK_CONTAINER (editor), widget);
+	editor = gtk_type_new (bonobo_peditor_get_type ());
 
 	if (tc != CORBA_OBJECT_NIL)
 		editor->tc = (CORBA_TypeCode)CORBA_Object_duplicate 
 			((CORBA_Object) tc, NULL);
 
+	editor->priv->widget = widget;
 	editor->priv->set_cb = set_cb;
 
+	gtk_signal_connect_object (GTK_OBJECT (widget), "destroy",
+				   bonobo_peditor_destroy,
+				   GTK_OBJECT (editor));
+
 	return editor;
 }
 
 void                
-bonobo_property_editor_set_value (BonoboPropertyEditor *editor,
-				  const BonoboArg      *value,
-				  CORBA_Environment    *opt_ev)
+bonobo_peditor_set_value (BonoboPEditor     *editor,
+			  const BonoboArg   *value,
+			  CORBA_Environment *opt_ev)
 {
-	CORBA_Environment  ev, *my_ev;
+	CORBA_Environment ev, *my_ev;
 	CORBA_any nv;
 
 	bonobo_return_if_fail (editor != NULL, opt_ev);
-	bonobo_return_if_fail (BONOBO_IS_PROPERTY_EDITOR(editor), opt_ev);
+	bonobo_return_if_fail (BONOBO_IS_PEDITOR(editor), opt_ev);
 	bonobo_return_if_fail (value != NULL, opt_ev);
 
 	if (!editor->priv->property)
@@ -393,13 +371,22 @@ bonobo_property_editor_set_value (BonoboPropertyEditor *editor,
 		CORBA_exception_free (&ev);
 }
 
+GtkWidget *
+bonobo_peditor_get_widget (BonoboPEditor *editor) 
+{
+	g_return_val_if_fail (editor != NULL, NULL);
+	g_return_val_if_fail (BONOBO_IS_PEDITOR (editor), NULL);
+
+	return editor->priv->widget;
+}
+
 typedef struct {
 	CORBA_TypeCode tc;
-	GtkWidget *(*new) (CORBA_TypeCode tc);
+	GtkObject *(*new) (CORBA_TypeCode tc);
 } PEPlugin;
 
-GtkWidget *
-bonobo_property_editor_resolve (CORBA_TypeCode tc)
+GtkObject *
+bonobo_peditor_resolve (CORBA_TypeCode tc)
 {
 	static GHashTable *pehash = NULL;
 	PEPlugin *plugin;
@@ -412,13 +399,13 @@ bonobo_property_editor_resolve (CORBA_TypeCode tc)
 
 		plugin = g_new0 (PEPlugin, 1);
 		plugin->tc = TC_Bonobo_Config_FileName;
-		plugin->new = bonobo_property_editor_filename_new;
+		plugin->new = bonobo_peditor_filename_new;
 		g_hash_table_insert (pehash, (gpointer)plugin->tc->repo_id, 
 				     plugin);
 
 		plugin = g_new0 (PEPlugin, 1);
 		plugin->tc = TC_Bonobo_Config_Color;
-		plugin->new = bonobo_property_editor_color_new;
+		plugin->new = bonobo_peditor_color_new;
 		g_hash_table_insert (pehash, (gpointer)plugin->tc->repo_id, 
 				     plugin);
 	}
@@ -429,55 +416,55 @@ bonobo_property_editor_resolve (CORBA_TypeCode tc)
 	switch (tc->kind) {
 
 	case CORBA_tk_short:    
-		return bonobo_property_editor_short_new ();
+		return bonobo_peditor_short_new ();
 	case CORBA_tk_long:   
-		return bonobo_property_editor_long_new ();
+		return bonobo_peditor_long_new ();
 	case CORBA_tk_ushort: 
-		return bonobo_property_editor_ushort_new ();
+		return bonobo_peditor_ushort_new ();
 	case CORBA_tk_ulong: 
-		return bonobo_property_editor_ulong_new ();
+		return bonobo_peditor_ulong_new ();
 	case CORBA_tk_float: 
-		return bonobo_property_editor_float_new ();
+		return bonobo_peditor_float_new ();
 	case CORBA_tk_double: 
-		return bonobo_property_editor_double_new ();
+		return bonobo_peditor_double_new ();
 	case CORBA_tk_string: 
-		return bonobo_property_editor_string_new ();
+		return bonobo_peditor_string_new ();
 	case CORBA_tk_boolean: 
-		return bonobo_property_editor_boolean_new (NULL);
+		return bonobo_peditor_boolean_new (NULL);
 	case CORBA_tk_enum:
-		return bonobo_property_editor_enum_new ();
+		return bonobo_peditor_enum_new ();
 	case CORBA_tk_char: 
-		// return bonobo_property_editor_char_new ();
+		// return bonobo_peditor_char_new ();
 	case CORBA_tk_struct:
-		// return bonobo_property_editor_struct_new (tc);
+		// return bonobo_peditor_struct_new (tc);
 	case CORBA_tk_sequence:
-		// return bonobo_property_editor_list_new (tc);
+		// return bonobo_peditor_list_new (tc);
 	case CORBA_tk_array:  
-		// return bonobo_property_editor_array_new (tc);
+		// return bonobo_peditor_array_new (tc);
 	default:              
-		return bonobo_property_editor_default_new ();
+		return bonobo_peditor_default_new ();
 	}
 }
 
-GtkWidget *
-bonobo_property_editor_new (Bonobo_PropertyBag  pb,
-			    const char         *name,
-			    CORBA_TypeCode      tc, 
-			    CORBA_any          *defval)
+GtkObject *
+bonobo_peditor_new (Bonobo_PropertyBag  pb,
+		    const char         *name,
+		    CORBA_TypeCode      tc, 
+		    CORBA_any          *defval)
 {
-	GtkWidget *w;
+	GtkObject *o;
 
 	g_return_val_if_fail (pb != CORBA_OBJECT_NIL, NULL);
 	g_return_val_if_fail (name != NULL, NULL);
 	g_return_val_if_fail (tc != CORBA_OBJECT_NIL, NULL);
 
-	if (!(w = bonobo_property_editor_resolve (tc))) 
+	if (!(o = bonobo_peditor_resolve (tc))) 
 		return NULL;
 
-	bonobo_property_editor_set_property (BONOBO_PROPERTY_EDITOR (w),
+	bonobo_peditor_set_property (BONOBO_PEDITOR (o),
 					     pb, name, tc, defval);
 
-	return w;
+	return o;
 }
 
 static void
@@ -510,9 +497,9 @@ remove_listener_cb (GtkWidget *widget, Bonobo_PropertyBag bag)
 }
 
 void
-bonobo_property_editor_set_guard (GtkWidget          *widget,
-				  Bonobo_PropertyBag  bag,
-				  const char         *prop_name)
+bonobo_peditor_set_guard (GtkWidget          *widget,
+			  Bonobo_PropertyBag  bag,
+			  const char         *prop_name)
 {
 	CORBA_Environment ev;
 	char *mask;
